## USAGE: cl < 01_prepcalib.cl
##
#
#         NOTICE: cl will not run a script containing CR+LF at line ends
#                 but exit with a syntax error. Use e.g. dos2unix in this case.
# IRAF general:  "e" or "ehist" will give you a history with cursor up/down
#                delete-key may act as backspace in xgterm
#                "help ccdlist" helps for routines
#                "?"  at prompts in graphics window shows options
#
!echo "<< This is TIGREleap 01_prepcalib.cl"
time

## Activate required IRAF packages  (this doesn't seem to work automatically)
images
imutil
noao
onedspec
imred
ccdred
twodspec
apextract

## Set output format 
#  (flpr "flush process cache" ... long story ... maybe needed to activate it)
#set imtype = "fits"
if (defvar("rawdir"))
   set imtype = "fits"
else
   error (1, "Please set the 'rawdir' variable\n")

flpr

## set orientation of dispersion axis for package twodspec.apextract
apextract.dispaxis = 2

## header translation file for ccdred
! rm -f "tigre.dat"
copy ("tigreleap$tigre.dat", "tigre.dat")
setinst.site=""
setinst.dir=""
setinstr ("tigre", review-, query-)

! rm -f log.txt
! rm -f tmp*.fits

# -- Creating lists --
#
!echo "  * Creating list and gathering image statistics"
! rm -f lst.tmp

ccdlist ("rawdir$*.fits", >"lst.tmp")

# Get list of flat field images
#
if (defvar("tigre_master_flat")) {
   ! echo "<< FF master image: ${tigre_master_flat}:" >>log.txt
} else {
   ! test -f lst.flat.1 && cp lst.flat.1 lst.flat.1.bck
   ! cat lst.tmp | grep 'Flat$' | sed -e 's/\[.*//g' >lst.flat.1
   !echo "<< Flatfield images (lst.flat.1):" >>log.txt
   ! head -n 20 lst.flat.1 >lst.flat.1.head
   ! tail -n 20 lst.flat.1 >lst.flat.1.tail
   imstat ("@lst.flat.1", >>"log.txt")
} 


# Get list of bias images
#
! test -f lst.bias.1 && rm lst.bias.1
! cat lst.tmp | grep 'Bias$' | sed -e 's/\[.*//g' >lst.bias.1
!echo "<< Bias images (lst.bias.1):" >>log.txt
imstat ("@lst.bias.1", >>"log.txt")

# Get list of dark images, only use first three (known burn-in effect for last three)
#
! test -f lst.dark.1 && rm lst.dark.1
! cat lst.tmp | grep 'Dark$' | sed -e 's/\[.*//g' | head -n 3 >lst.dark.1
!echo "<< Dark images (lst.dark.1):" >>log.txt
imstat ("@lst.dark.1", >>"log.txt")

# list of arc images
#
if (defvar("tigre_master_thar")) {
   ! echo "<< ThAr master image: ${tigre_master_thar}:" >>log.txt
} else {
   ! test -f lst.arc.1 && rm lst.arc.1
   ! cat lst.tmp | grep 'ThAr$' | sed -e 's/\[.*//g' >lst.arc.1
   !echo "<< ThAr images (lst.arc.1):" >>log.txt
   imstat ("@lst.arc.1", >>"log.txt")
}

# Get list of science images
#
! rm -f lst.science.0
ccdlist ("rawdir$Sci_*.fits", >"lst.science.0")
! test -f lst.science.1 && rm lst.science.1
! cat lst.science.0 | sed -e 's/\[.*//g' >lst.science.1
!echo "<< Science images (lst.science.1):" >>log.txt
imstat ("@lst.science.1", >>"log.txt")
! rm -f lst.science.0
#
time


# -- Prepare Bias (ical.bias) --
#
!echo "  * Preparing master bias"
#
# combine using the known CCD noise parameters, reject positive excursions only
# (tested to give better stdev than avsigclip)
imdel ("ical.bias.fits", go_ahead+, verify-)
imdel ("ical.bia0.fits", go_ahead+, verify-)
#
combine    ("@lst.bias.1", output="ical.bia0", comb="average", reject="crreject", ccdtype="", rdnoise="RDNOISE", gain="GAIN", subsets-, del-, scale="none", stat="", mclip+, lsig=3., hsig=3., nkeep=1)
blkavg     ("ical.bia0", "tmp1", 1, 2048)
fit1d      ("tmp1", "tmp2", "fit", order=40, func="chebyshev", interact-, axis=1, sample="*")
blkrep     ("tmp2", "ical.bias", 1, 2048)
imdel      ("tmp1")
imdel      ("tmp2")
#
blkavg     ("ical.bia0", "tmp1", 2088, 1)
fit1d      ("tmp1[1,*]", "tmp2", "fit", order=3, func="chebyshev", interact-, axis=1)
blkrep     ("tmp2", "tmp3", 2088, 1)
imstat     ("tmp3", fields="mean", format-) | scan(x)
imarith    ("tmp3", "-", x, "tmp4")
imarith    ("ical.bias", "+", "tmp4", "ical.bias")
imdel      ("tmp1")
imdel      ("tmp2")
imdel      ("tmp3")
imdel      ("tmp4")
#
imdel      ("check.bias")
imarith    ("ical.bia0", "-", "ical.bias", "check.bias")
imstat     ("check.bias", >>"log.txt")
#
time


# -- Prepare Dark (ical.dark) --
#
!echo "  * Preparing master dark"
#
imdel     ("ical.dark.fits", go_ahead+, verify-)
imdel     ("ical.dar0.fits", go_ahead+, verify-)
imdel     ("ical.dar1.fits", go_ahead+, verify-)
imdel     ("ical.dar2.fits", go_ahead+, verify-)
#
combine   ("@lst.dark.1", output="ical.dar0", comb="median",  reject="crreject", ccdtype="", rdnoise="RDNOISE", gain="GAIN", subsets-, del-, scale="none", stat="", mclip+, lsig=3., hsig=3., nkeep=1)
#
ccdproc    ("ical.dar0", output="ical.dark", ccdtype="", fixpix-, overscan-, trim+, zerocor+, darkcor-, flatcor-, illumcor-, fringecor-, readcor-, scancor-, trimsec="[22:2069,1:2048]", zero="ical.bias", dark="", minreplace=1.)
imcopy    ("ical.dark", "ical.dar1", verb-)
#
#imstat    ("ical.dark[1:20,1:2048]", fields="midpt", format-) | scan(x)
#imreplace ("ical.dark[1:20,1:2048]", x)
#imstat    ("ical.dark[2070:2088,1:2048]", fields="midpt", format-) | scan(x)
#imreplace ("ical.dark[2070:2088,1:2048]", x)
#imstat    ("ical.dark[21:2069,1:512]", fields="midpt", format-) | scan(x)
#imreplace ("ical.dark[21:2069,1:512]", x)
#imstat    ("ical.dark[21:2069,513:2048]", fields="midpt", format-) | scan(x)
#imreplace ("ical.dark[21:2069,513:2048]", x)
#
imstat    ("ical.dark[1:2048,1:512]", fields="midpt", format-) | scan(x)
imreplace ("ical.dark[1:2048,1:512]", x)
imstat    ("ical.dark[1:2048,513:2048]", fields="midpt", format-) | scan(x)
imreplace ("ical.dark[1:2048,513:2048]", x)
#
imdel     ("check.dark0")
imarith   ("ical.dar1", "-", "ical.dark", "check.dark0")
imsurfit  ("check.dark0", "tmp1", 5, 10, type="fit", function="spline3", xmed=20, ymed=20)
imcopy    ("ical.dark", "ical.dar2", verb-)
imarith   ("ical.dark", "+", "tmp1", "ical.dark")
#
imdel     ("check.dark1")
imarith   ("ical.dar1", "-", "ical.dark", "check.dark1")
median    ("check.dark1", "tmp2", 9, 9, boundary="near", verb-)
boxcar    ("tmp2", "tmp3", 64, 64)
imarith   ("ical.dark", "+", "tmp3", "ical.dark")
#
imdel     ("check.dark")
imarith   ("ical.dar1", "-", "ical.dark", "check.dark")
imdel     ("tmp1")
imdel     ("tmp2")
imdel     ("tmp3")
imstat    ("check.dark0", >>"log.txt")
imstat    ("check.dark1", >>"log.txt")
imstat    ("check.dark",  >>"log.txt")
#
time

# -- Prepare Flat (ical.flat) --
#
!echo "  * Preparing master flat"
#
imdel ("ical.fla0.fits", go_ahead+, verify-)
imdel ("ical.flat.fits", go_ahead+, verify-)
#
imdel ("ical.fla0.head.fits", go_ahead+, verify-)
imdel ("ical.flat.head.fits", go_ahead+, verify-)
#
imdel ("ical.fla0.tail.fits", go_ahead+, verify-)
imdel ("ical.flat.tail.fits", go_ahead+, verify-)
#
if (defvar("tigre_master_flat")) {
   s2 = envget("tigre_master_flat")
   imcopy (s2, "ical.fla0", >>"log.txt")
   imcopy (s2, "ical.fla0.head", >>"log.txt")
   imcopy (s2, "ical.fla0.tail", >>"log.txt")
} else {

  # nkeep=-3, scale=median to make sure not too many points are dropped (master flatfield quality was bad initially)
  #
  combine    ("@lst.flat.1", output="ical.fla0", comb="average", reject="avsigclip", ccdtype="", subsets-, del-, scale="median", zero="none", stat="", mclip+, lsig=3., hsig=3., nkeep=-3)
  #
  combine    ("@lst.flat.1.head", output="ical.fla0.head", comb="average", reject="avsigclip", ccdtype="", subsets-, del-, scale="median", zero="none", stat="", mclip+, lsig=3., hsig=3., nkeep=-3)
  #
  combine    ("@lst.flat.1.tail", output="ical.fla0.tail", comb="average", reject="avsigclip", ccdtype="", subsets-, del-, scale="median", zero="none", stat="", mclip+, lsig=3., hsig=3., nkeep=-3)
  #
  ccdproc    ("ical.fla0*", output="", ccdtype="", fixpix-, overscan-, trim+, zerocor+, darkcor+, flatcor-, illumcor-, fringecor-, readcor-, scancor-, trimsec="[22:2069,1:2048]", zero="ical.bias", dark="ical.dark", minreplace=1.)
}
#
apscat1.function = "spline3"
apscat1.order = 1
apscat1.sample = "*"
apscat1.naverage = 1
apscat1.low_reject = 5.
apscat1.high_reject = 1.5
apscat1.niterate = 5
apscat1.grow = 1.
#
apscat2.function = "spline3"
apscat2.order = 9
apscat2.sample = "*"
apscat2.naverage = 1
apscat2.low_reject = 3.
apscat2.high_reject = 3.
apscat2.niterate = 0
apscat2.grow = 0.
#
i = strlen(envget("rawdir"))
s1 = strlwr(substr(envget("rawdir"), i-1, i-1))
if (s1 == "r") {
   !test -d database || mkdir database
   imdel ("tigre.r")
   imcopy ("tigreleap$tigre.r", "tigre.r")
   copy ("tigreleap$database/aptigre.r", "database/aptigre.r")
   s2="tigre.r"
} else {
   !test -d database || mkdir database
   imdel ("tigre.b")
   imcopy ("tigreleap$tigre.b", "tigre.b")
   copy ("tigreleap$database/aptigre.b", "database/aptigre.b")
   s2="tigre.b"

   apscat1.order = 9
}
imdel("check.scatter")
apscatter ("ical.fla0", "ical.flat", scatter="check.scatter", refer=s2, inter-, find-, recenter-, resize-, edit-, trace-, fittr-)
#
apscatter ("ical.fla0.head", "ical.flat.head", scatter="", refer=s2, inter-, find-, recenter-, resize-, edit-, trace-, fittr-)
apscatter ("ical.fla0.tail", "ical.flat.tail", scatter="", refer=s2, inter-, find-, recenter-, resize-, edit-, trace-, fittr-)
#
time


# -- Prepare ThAr (ical.thar) --
#
!echo "  * Preparing master thar"
#
imdel ("ical.thar.fits", go_ahead+, verify-)
#
if (defvar("tigre_master_thar")) {
   s2 = envget("tigre_master_thar")
   imcopy (s2, "ical.thar", >>"log.txt")
} else {
  combine    ("@lst.arc.1", output="ical.thar", comb="average", reject="avsigclip", ccdtype="", subsets-, del-, scale="none", stat="", mclip+, lsig=3., hsig=3., nkeep=1)
  #
  ccdproc    ("ical.thar", output="", ccdtype="", fixpix-, overscan-, trim+, zerocor+, darkcor+, flatcor-, illumcor-, fringecor-, readcor-, scancor-, trimsec="[22:2069,1:2048]", zero="ical.bias", dark="ical.dark", minreplace=1.)
  #
}
time
