## USAGE: cl < 02_extractcalib.cl
##
#
#         NOTICE: cl will not run a script containing CR+LF at line ends
#                 but exit with a syntax error. Use e.g. dos2unix in this case.
# IRAF general:  "e" or "ehist" will give you a history with cursor up/down
#                delete-key may act as backspace in xgterm
#                "help ccdlist" helps for routines
#                "?"  at prompts in graphics window shows options
#
!echo "<< This is TIGREleap 02_extractcalib.cl"
time

## Activate required IRAF packages  (this doesn't seem to work automatically)
images
imutil
noao
imred
echelle
ccdred
twodspec
apextract
onedspec
astutil

## Set output format 
#  (flpr "flush process cache" ... long story ... maybe needed to activate it)
#set imtype = "fits"
if (defvar("rawdir"))
   set imtype = "fits"
else
   error (1, "Please set the 'rawdir' variable\n")

flpr

## set orientation of dispersion axis for package twodspec.apextract
apextract.dispaxis = 2

## header translation file for ccdred
! rm -f "tigre.dat"
copy ("tigreleap$tigre.dat", "tigre.dat")
setinst.site=""
setinst.dir=""
setinstr ("tigre", review-, query-)

## Red or Blue Channel
i = strlen(envget("rawdir"))
s1 = strlwr(substr(envget("rawdir"), i-1, i-1))
if (s1 == "r") {
   !test -d database || mkdir database

   imdel ("tigre.r", ver-)
   imdel ("tigre.r.thar.ec", ver-)
   imcopy ("tigreleap$tigre.r", "tigre.r")
   imcopy ("tigreleap$tigre.r.thar.ec", "tigre.r.thar.ec")
   copy ("tigreleap$database/aptigre.r", "database/aptigre.r")
   copy ("tigreleap$database/ectigre.r.thar.ec", "database/ectigre.r.thar.ec")

   s2="tigre.r"
   s3="tigre.r.thar.ec"
   b1=no

} else {
   !test -d database || mkdir database

   imdel ("tigre.b", ver-)
   imdel ("tigre.b.thar.ec", ver-)
   imcopy ("tigreleap$tigre.b", "tigre.b")
   imcopy ("tigreleap$tigre.b.thar.ec", "tigre.b.thar.ec")
   copy ("tigreleap$database/aptigre.b", "database/aptigre.b")
   copy ("tigreleap$database/ectigre.b.thar.ec", "database/ectigre.b.thar.ec")

   s2="tigre.b"
   s3="tigre.b.thar.ec"
   b1=yes
}

! rm -f tmp*.fits

#
# ---- extract calibration spectra ----
#

!echo "  * Extracting calibration spectra"
#
aprecenter ("ical.flat", apertures="", refer=s2,          inter-, find-, recenter+, resize-, edit-, nsum=30, line=1000, aprecenter="", npeak=24, shift+)
apresize   ("ical.flat", apertures="", refer="",          inter-, find-, recenter-, resize+, edit-, nsum=30, line=1000, llim=INDEF, ulim=INDEF, ylev=0.05, peak+, bkg+, r_grow=0.,  avglimits-)
#
aprecenter ("ical.flat.head", apertures="", refer=s2,          inter-, find-, recenter+, resize-, edit-, nsum=30, line=1000, aprecenter="", npeak=24, shift+)
apresize   ("ical.flat.head", apertures="", refer="",          inter-, find-, recenter-, resize+, edit-, nsum=30, line=1000, llim=INDEF, ulim=INDEF, ylev=0.05, peak+, bkg+, r_grow=0.,  avglimits-)
#
aprecenter ("ical.flat.tail", apertures="", refer=s2,          inter-, find-, recenter+, resize-, edit-, nsum=30, line=1000, aprecenter="", npeak=24, shift+)
apresize   ("ical.flat.tail", apertures="", refer="",          inter-, find-, recenter-, resize+, edit-, nsum=30, line=1000, llim=INDEF, ulim=INDEF, ylev=0.05, peak+, bkg+, r_grow=0.,  avglimits-)
#
# Flatfield orders
#
imdel      ("ical.flat.ec", ver-)
imdel      ("ical.flat.head.ec", ver-)
imdel      ("ical.flat.tail.ec", ver-)
#
!echo "  * Extracting flatfield orders"
#
apsum      ("ical.flat", apertures="", refer="",          out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras-, review-, back="none", weights="none", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=4., usig=4., nsub=1)
#
apsum      ("ical.flat.head", apertures="", refer="",          out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras-, review-, back="none", weights="none", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=4., usig=4., nsub=1)
#
apsum      ("ical.flat.tail", apertures="", refer="",          out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras-, review-, back="none", weights="none", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=4., usig=4., nsub=1)
#
# ThAr orders
#
!echo "  * Extracting thar orders"
#
imdel      ("ical.thar.ec", ver-)
apsum      ("ical.thar", apertures="", refer="ical.flat", out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras-, review-, back="none", weights="none", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=4., usig=4., nsub=1, line=INDEF, nsum=50)
#
!echo "  * Extracting thar orders"
#
ecreidentify  ("ical.thar.ec", refer=s3, shift=INDEF, cradius=2., thresh=10., refit+, database="database")

#
# ---- continuum fit ----
#

imdel      ("ical.flat_c.ec", ver-)
imdel      ("ical.flat_d.ec", ver-)
imdel      ("ical.flat_t.ec", ver-)
imdel      ("flat.sc", ver-)
#
# continuum  ("ical.flat.ec", "ical.flat_c.ec", "NO", typ="fit", interact-, function="spline3", order=11, low_rej=2.0, high_rej=0., niter=10, grow=1.)
continuum  ("ical.flat.ec", "ical.flat_c.ec", "NO", typ="fit", interact-, function="chebyshev", order=7, low_rej=1.5, high_rej=0., niter=10, grow=2.)
refspectra  ("ical.flat_c.ec", answer="yes", refer="ical.thar.ec", apert="", refaps="", select="nearest", group="none", ignoreaps+, override+, confirm-, assign+, verb-)
dispcor    ("ical.flat_c.ec", "ical.flat_d.ec", linear+, database="database", table="", w1=INDEF, w2=INDEF, dw=INDEF, nw=INDEF, log-, flux+, blank=0., samedisp-, global-, ignoreaps-, confirm-, listonly-, verbose-)

#
# ---- order merging [b1=yes is BLUE]; 1600 -> 1500
#
if (b1) {
   # imcopy     ("ical.flat_d.ec[501:1500,*]", "ical.flat_t.ec")
   imcopy     ("ical.flat_d.ec[601:1500,*]", "ical.flat_t.ec")
   # imreplace  ("ical.flat_t.ec[840:1000,1]", 1.0)
   imreplace  ("ical.flat_t.ec[740:900,1]", 1.0)
   scombine   ("ical.flat_t.ec", "flat.sc", group="images", combine="sum", reject="none", >>"/dev/null")
} else {
   imcopy     ("ical.flat_d.ec[251:1600,*]", "ical.flat_t.ec")
   scombine   ("ical.flat_t.ec", "flat.sc", group="images", combine="sum", reject="none", >>"/dev/null")
}
#
time

