## USAGE: cl < 03_extractspec.cl
##
#
#         NOTICE: cl will not run a script containing CR+LF at line ends
#                 but exit with a syntax error. Use e.g. dos2unix in this case.
# IRAF general:  "e" or "ehist" will give you a history with cursor up/down
#                delete-key may act as backspace in xgterm
#                "help ccdlist" helps for routines
#                "?"  at prompts in graphics window shows options
#
!echo "<< This is TIGREleap 03_extractspec.cl"
time

## Activate required IRAF packages  (this doesn't seem to work automatically)
images
imutil
noao
imred
echelle
ccdred
twodspec
apextract
onedspec
astutil

## Set output format 
#  (flpr "flush process cache" ... long story ... maybe needed to activate it)
#set imtype = "fits"
if (defvar("rawdir"))
   set imtype = "fits"
else
   error (1, "Please set the 'rawdir' variable\n")

flpr

## set orientation of dispersion axis for package twodspec.apextract
apextract.dispaxis = 2

## header translation file for ccdred
! rm -f "tigre.dat"
copy ("tigreleap$tigre.dat", "tigre.dat")
setinst.site=""
setinst.dir=""
setinstr ("tigre", review-, query-)

## Red or Blue Channel
i = strlen(envget("rawdir"))
s1 = strlwr(substr(envget("rawdir"), i-1, i-1))
if (s1 == "r") {
   !test -d database || mkdir database

   imdel ("tigre.r", ver-)
   imdel ("tigre.r.thar.ec", ver-)
   imcopy ("tigreleap$tigre.r", "tigre.r")
   imcopy ("tigreleap$tigre.r.thar.ec", "tigre.r.thar.ec")
   copy ("tigreleap$database/aptigre.r", "database/aptigre.r")
   copy ("tigreleap$database/ectigre.r.thar.ec", "database/ectigre.r.thar.ec")

   s2="tigre.r"
   s3="tigre.r.thar.ec"
   b1=no

} else {
   !test -d database || mkdir database

   imdel ("tigre.b", ver-)
   imdel ("tigre.b.thar.ec", ver-)
   imcopy ("tigreleap$tigre.b", "tigre.b")
   imcopy ("tigreleap$tigre.b.thar.ec", "tigre.b.thar.ec")
   copy ("tigreleap$database/aptigre.b", "database/aptigre.b")
   copy ("tigreleap$database/ectigre.b.thar.ec", "database/ectigre.b.thar.ec")

   s2="tigre.b"
   s3="tigre.b.thar.ec"
   b1=yes
}

! rm -f tmp*.fits


#
# ---- extract science spectra ----
#
!echo "  * Extracting science spectra"
#
!cat lst.science.1 | sed 's/.fits/_b.ec/g'   | sed 's/rawdir\$//g' >lst.sci_b
!cat lst.science.1 | sed 's/.fits/_c.ec/g'   | sed 's/rawdir\$//g' >lst.sci_c
!cat lst.science.1 | sed 's/.fits/_d.ec/g'   | sed 's/rawdir\$//g' >lst.sci_d
#
!cat lst.science.1 | sed 's/.fits//g' | sed 's/rawdir\$//g' >lst.tmp
list = "lst.tmp"
#
apscat1.function = "spline3"
apscat1.order = 15
apscat1.sample = "*"
apscat1.naverage = 1
apscat1.low_reject = 5.
apscat1.high_reject = 1.5
apscat1.niterate = 5
apscat1.grow = 1.
#
apscat2.function = "spline3"
apscat2.order = 9
apscat2.sample = "*"
apscat2.naverage = 1
apscat2.low_reject = 3.
apscat2.high_reject = 3.
apscat2.niterate = 0
apscat2.grow = 0.
#
while (fscan(list, s1) != EOF)
{
        imdel(s1//"_a", ver-)
	imdel(s1//"_b", ver-)
        imdel(s1//"_b.ec", ver-)
        imdel(s1//"_c.ec", ver-)
	imdel(s1//"_sigma.ec", ver-)
        imdel(s1//"_d.ec", ver-)
	imdel(s1//"_orders.ec", ver-)
        imdel(s1//"_t.ec", ver-)
	imdel(s1//"_tm.ec", ver-)
	imdel(s1//".sc",  ver-)
	imdel(s1//".merged",  ver-)

        imdel(s1//"_cs.ec", ver-)
        imdel(s1//"_ds.ec", ver-)
	imdel(s1//"_orders_S.ec", ver-)
        imdel(s1//"_ts.ec", ver-)
	imdel(s1//"_tms.ec", ver-)
	imdel(s1//".scs", ver-)
	imdel(s1//".merged_S", ver-)

	ccdproc    ("rawdir$"//s1, output=s1//"_a", ccdtype="", fixpix-, overscan-, trim+, zerocor+, darkcor+, flatcor-, illumcor-, fringecor-, readcor-, scancor-, trimsec="[22:2069,1:2048]", zero="ical.bias", dark="ical.dark", minreplace=1.)

	# for rvcorr
	#
	hselect ("rawdir$"//s1, "DATE", "yes") | scan(s2)
	if (0 != fscanf  (s2, "%2d-%2d-%4d", i, j, k))
	   printf  ("%04d-%02d-%02d", k, j, i) | scan(s2)
	hselect ("rawdir$"//s1, "UT", "yes") | scan(s3)
	s2 = s2//"T"//s3
	hedit   (s1//"_a", "DATE-OBS", s2, add+, addonly+, del-, ver-, show-, update+)

	hselect ("rawdir$"//s1, "RA_TEL", "yes") | scan(s2)
	# use "dummy" to establish string type
	hedit   (s1//"_a", "RA","dummy", add+, addonly+, del-, ver-, show-, update+)
	hedit   (s1//"_a", "RA", s2, add-, addonly-, del-, ver-, show-, update+)
	hselect ("rawdir$"//s1, "DEC_TEL", "yes") | scan(s2)
	hedit   (s1//"_a", "DEC", "dummy", add+, addonly+, del-, ver-, show-, update+)
	hedit   (s1//"_a", "DEC", s2, add-, addonly-, del-, ver-, show-, update+)
	hedit   (s1//"_a", "EPOCH", 2000., add+, addonly+, del-, ver-, show-, update+)

	# for rvcorr
	hedit   (s1//"_a", "OBSERVAT", "tigre", add-, addonly-, del-, ver-, show-, update+)
	rvcorr  (image=s1//"_a", epoch=2000., header+, input+, imupdate+, >>"/dev/null")
	hedit   (s1//"_a", "OBSERVAT", "Observatorio La Luz", add-, addonly-, del-, ver-, show-, update+)

	apscatter (s1//"_a", output=s1//"_b", refer="ical.flat", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, subtract+, smooth+, fitsmooth-, line=INDEF, nsum=10, buffer=1.)

	# adjust rdnoise (5.81) upward to account for apscatter subtraction

	# apresize.ylevel = 0.1
	# apresize.bkg    = no

	apresize   (s1//"_b", apertures="", refer="ical.flat", inter-,  find-, recenter-, resize+, edit-, line=INDEF, nsum=50, llimit=-8, ulimit=8, ylevel=0.1, peak+, bkg-, r_grow=0., avg+)

	# use lsig=18. because fringing distorts profile, which confuses the CLEAN routine
	# (we would need to do 2D flatfielding to do this correctly...)

	# apsum [b1 == yes is BLUE]

	if (b1) {
	   apsum      (s1//"_b", apertures="", refer="", out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras+, review-, back="none", weights="variance", pfit="fit1d", clean+, satur=63000, readnoise="12.", gain="GAIN", lsig=3., usig=3., nsub=1, line=INDEF, nsum=50)
	} else {
	   apsum      (s1//"_b", apertures="", refer="", out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras+, review-, back="none", weights="variance", pfit="fit1d", clean+, satur=63000, readnoise="12.", gain="GAIN", lsig=18., usig=4., nsub=1, line=INDEF, nsum=50)
	}

	# prepare (extract) flat field

	imdel      ("ical.flat_tmp", ver-)
	imdel      ("ical.flat_tmp.ec", ver-)
	imcopy     ("ical.flat", "ical.flat_tmp", >>"/dev/null")
	#
	if (b1) {
	  apsum      ("ical.flat_tmp", apertures="", refer=s1//"_b", out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras+, review-, back="none", weights="variance", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=3., usig=3., nsub=1)
	} else {
	  apsum      ("ical.flat_tmp", apertures="", refer=s1//"_b", out="", format="echelle", inter-,  find-, recenter-, resize-, edit-, trace-, fittr-, extract+, extras+, review-, back="none", weights="variance", pfit="fit1d", clean-, satur=INDEF, readnoise="RDNOISE", gain="GAIN", lsig=18., usig=4., nsub=1)
	}

	# divide by flat field, do not use imdiv rescale because cannot define stat section

	imdivide   (s1//"_b.ec[*,*,1]", "ical.flat_tmp.ec[*,*,1]", s1//"_c.ec", title="*", const=0., rescale="norescale", verb-)
	imstat     (s1//"_b.ec[500:1600,*,1]", fields="midpt", format-) | scan(x)
	imstat     (s1//"_c.ec[500:1600,*]", fields="midpt", format-) | scan(y)
	z = (x / y)
	imarith    (s1//"_c.ec", "*", z, s1//"_c.ec")

	imdivide   (s1//"_b.ec[*,*,2]", "ical.flat_tmp.ec[*,*,2]", s1//"_cs.ec", title="*", const=0., rescale="norescale", verb-)
	imstat     (s1//"_b.ec[500:1600,*,2]", fields="midpt", format-) | scan(x)
	imstat     (s1//"_cs.ec[500:1600,*]", fields="midpt", format-) | scan(y)
	z = (x / y)
	imarith    (s1//"_cs.ec", "*", z, s1//"_cs.ec")

	imcopy     (s1//"_b.ec[*,*,3]",  s1//"_sigma.ec", >>"/dev/null")

	# save flat field for inspection

	imdel      (s1//"_flat.ec", ver-)
	imcopy     ("ical.flat_tmp.ec", s1//"_flat.ec", >>"/dev/null")

	# apply wavelength calibration

	refspectra  (s1//"_c.ec", answer="yes", refer="ical.thar.ec", apert="", refaps="", select="nearest", group="none", ignoreaps+, override+, confirm-, assign+, verb-)
	refspectra  (s1//"_cs.ec", answer="yes", refer="ical.thar.ec", apert="", refaps="", select="nearest", group="none", ignoreaps+, override+, confirm-, assign+, verb-)

	dispcor    (s1//"_c.ec", s1//"_d.ec", linear+, database="database", table="", w1=INDEF, w2=INDEF, dw=INDEF, nw=INDEF, log-, flux+, blank=0., samedisp-, global-, ignoreaps-, confirm-, listonly-, verbose-)
	dispcor    (s1//"_cs.ec", s1//"_ds.ec", linear+, database="database", table="", w1=INDEF, w2=INDEF, dw=INDEF, nw=INDEF, log-, flux+, blank=0., samedisp-, global-, ignoreaps-, confirm-, listonly-, verbose-)

	imcopy     (s1//"_d.ec",  s1//"_orders.ec",   >>"/dev/null")
	imcopy     (s1//"_ds.ec", s1//"_orders_S.ec", >>"/dev/null")

	# order merging [b1 == yes is BLUE]; 1600 -> 1500

	if (b1) {
	   imcopy     (s1//"_d.ec[601:1500,*]", s1//"_t.ec", >>"/dev/null")
	   imarith    (s1//"_t.ec", "*", "ical.flat_t.ec", s1//"_tm.ec")
	   imreplace  (s1//"_tm.ec[740:900,1]", 1.0)
	
	   imcopy     (s1//"_ds.ec[601:1500,*]", s1//"_ts.ec", >>"/dev/null")
	   imarith    (s1//"_ts.ec", "*", "ical.flat_t.ec", s1//"_tms.ec")
	   imreplace  (s1//"_tms.ec[740:900,1]", 1.0)	
	} else {
	   imcopy     (s1//"_d.ec[251:1600,*]", s1//"_t.ec", >>"/dev/null")
	   imarith    (s1//"_t.ec", "*", "ical.flat_t.ec", s1//"_tm.ec")

	   imcopy     (s1//"_ds.ec[251:1600,*]", s1//"_ts.ec", >>"/dev/null")
	   imarith    (s1//"_ts.ec", "*", "ical.flat_t.ec", s1//"_tms.ec")
	}

	scombine   (s1//"_tm.ec", s1//".sc",  group="images", combine="sum", reject="none", >>"/dev/null")
	imdiv      (s1//".sc", "flat.sc", s1//".merged", const=0., rescale="norescale")

	scombine   (s1//"_tms.ec", s1//".scs",  group="images", combine="sum", reject="none", >>"/dev/null")
	imdiv      (s1//".scs", "flat.sc", s1//".merged_S", const=0., rescale="norescale")

	flprc
}
#
time
